;**********************************************************************
;                                                                     *
;    Filename:         MagiDavid AL101 - Lab 006.asm                  *
;    Date:             10/21/2011                                     *
;    File Version:     1.0                                            *
;                                                                     *
;    Author:           David Hoffman                                  *
;    Company:          NeoLoch, LLC                                   *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required: P16F882.INC                                      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    License: Creative Commons                                        *
;    Attribution-NonCommercial-ShareAlike 3.0 Unported License.       *
;             http://creativecommons.org/licenses/by-nc-sa/3.0        *
;                                                                     *
;**********************************************************************

     LIST      p=16F882            ; list directive to define processor
     #INCLUDE <p16F882.inc>        ; processor specific variable definitions

     __CONFIG _CONFIG1, _LVP_OFF & _FCMEN_OFF & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT 
     __CONFIG _CONFIG2, _WRT_OFF & _BOR40V

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.
; Note that the _DEBUG_ON is changed by selecting a debugger or programmer.  
; There isn't much advantage to setting it in the code.  

#DEFINE		DMD_ROW_PORT	PORTB
#DEFINE		DMD_ROW1	PORTB,0
#DEFINE		DMD_ROW2	PORTB,1
#DEFINE		DMD_ROW3	PORTB,2
#DEFINE		DMD_ROW4	PORTB,3
#DEFINE		DMD_ROW5	PORTB,4

#DEFINE		DMD_D1_BIT	PORTA,7
#DEFINE		DMD_D1_SET	BSF	DMD_D1_BIT
#DEFINE		DMD_D1_LATCH	BCF	DMD_D1_BIT

#DEFINE		DMD_D2_BIT	PORTB,5
#DEFINE		DMD_D2_SET	BSF	DMD_D2_BIT
#DEFINE		DMD_D2_LATCH	BCF	DMD_D2_BIT

#DEFINE		DMD_D3_BIT	PORTC,0
#DEFINE		DMD_D3_SET	BSF	DMD_D3_BIT
#DEFINE		DMD_D3_LATCH	BCF	DMD_D3_BIT

#DEFINE		DMD_REFRESH_RATE	0X01
#DEFINE		DMD_DATA_PORT	PORTA

DMD_VAR		UDATA
DMD_DISP_D1	RES 5			;RESERVE 5 MEMORY LOCATIONS FOR DIGIT 1.
DMD_DISP_D2	RES 5			;RESERVE 5 MEMORY LOCATIONS FOR DIGIT 2.
DMD_DISP_D3	RES 5			;RESERVE 5 MEMORY LOCATIONS FOR DIGIT 3.

DMD_VAR_SHR	UDATA_SHR
DMD_LOOP	RES 1
DMD_COUNT	RES 1
DMD_TEMP	RES 1
DMD_ROW_COUNT	RES 1
DMD_MTC_REFRESH	RES 1

;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

RESET_VECTOR    CODE   0x0000
          goto    START             ; When using debug header, ICD2 may not stop
                                    ; on instruction 0 during reset.

;------------------------------------------------------------------------------
; CONFIGURATION CODE
;------------------------------------------------------------------------------
;START	ORG	0X05
MAIN_PROG       CODE
START
	BANKSEL PORTA
	CLRF	PORTA

	BANKSEL	ANSEL
	CLRF	ANSEL

	BANKSEL	TRISA
	MOVLW	0X00
	MOVWF	TRISA

	CLRF	DMD_COUNT
	CLRF	DMD_LOOP

	BANKSEL	ANSELH			;SELECT THE BANK THAT ANSELH IS IN.
	CLRF	ANSELH			;SET PORTB AS DIGITAL I/O.
	BANKSEL	TRISB			;SELECT THE BANK THAT TRISB IS IN.
	CLRW
	MOVWF	TRISB			;SET PORT B AS DIGITAL OUTPUT.
	MOVWF	TRISC
	BANKSEL PORTB			;SELECT BANK THAT PORT B IS IN.
	MOVLW	0XFF			;COPY 0XFF TO W.
	MOVWF	PORTB			;TURN ALL PORTB BITS ON.

	MOVLW	DMD_REFRESH_RATE
	MOVWF	DMD_MTC_REFRESH
	CLRF	DMD_ROW_COUNT
	CLRF	DMD_DATA_PORT
	CLRF	DMD_ROW_PORT

	MOVLW	0XFF
	MOVWF	DMD_DISP_D2
;------------------------------------------------------------------------------
; SET INTERNAL OSCILLATOR TO 8 MHZ.
;------------------------------------------------------------------------------
	BANKSEL	OSCCON
	BSF	OSCCON,0


;------------------------------------------------------------------------------
; MAIN CODE
;------------------------------------------------------------------------------
MAIN
	BANKSEL	PORTA

	CALL	DMD_CHAR1
	CALL	DMD_UPDATE

	GOTO	MAIN			;LOOP AROUND AND IT ALL OVER AGAIN.



;------------------------------------------------------------------------------
; DOT MATRIX DISPLAY UPDATE CHECK
;------------------------------------------------------------------------------
DMD_UPDATE
	DECFSZ	DMD_MTC_REFRESH,F	;IS IT TIME TO UPDATE DISPLAY?
	RETURN				;NO
	MOVLW	DMD_REFRESH_RATE	;YES - RESET DMD_MTC_REFRESH TO DEFAULT
	MOVWF	DMD_MTC_REFRESH		;VALUE.

	INCF	DMD_ROW_COUNT,F		;GET NEXT ROW TO DISPLAY
	MOVF	DMD_ROW_COUNT,W		;COPY DATA TO W.
	ADDWF	PCL,F			;ADD TO PCL.
	NOP				;IGNORE PCL + 0
	GOTO	DMD_ROW1_UPDATE		;SHOW ROW 1 ON DISPLAY.
	GOTO	DMD_ROW2_UPDATE		;SHOW ROW 2 ON DISPLAY.
	GOTO	DMD_ROW3_UPDATE		;SHOW ROW 3 ON DISPLAY.
	GOTO	DMD_ROW4_UPDATE		;SHOW ROW 4 ON DISPLAY.
	GOTO	DMD_ROW5_UPDATE		;SHOW ROW 5 ON DISPLAY.
	CLRF	DMD_ROW_COUNT		;RESET ROW COUNTER TO 0.
	GOTO	DMD_UPDATE		;LOOP AROUND AND UPDATE ROW 1.
;------------------------------------------------------------------------------
; UPDATE ROW 1 THE DISPLAY
;------------------------------------------------------------------------------
DMD_ROW1_UPDATE
	BANKSEL	DMD_ROW_PORT
	MOVF	DMD_DISP_D1,W		;COPY DMD_DISP_D1 TO W.
	BCF	DMD_ROW5		;TURN OFF ROW 5.
	DMD_D1_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D1_LATCH			;LATCH 373 TO LOCK DATA.

	MOVF	DMD_DISP_D2,W		;COPY DMD_DISP_D2 TO W.
	DMD_D2_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D2_LATCH			;LATCH 373 TO LOCK DATA.

	MOVF	DMD_DISP_D3,W		;COPY DMD_DISP_D3 TO W.
	DMD_D3_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D3_LATCH			;LATCH 373 TO LOCK DATA.

	BSF	DMD_ROW1		;TURN ON ROW 1
	RETURN

;------------------------------------------------------------------------------
; UPDATE ROW 2 THE DISPLAY
;------------------------------------------------------------------------------
DMD_ROW2_UPDATE
	BANKSEL	DMD_ROW_PORT
	MOVF	DMD_DISP_D1+1,W		;COPY DMD_DISP_D1+1 TO W.
	BCF	DMD_ROW1		;TURN OFF ROW 1.
	DMD_D1_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D1_LATCH			;LATCH 373 TO LOCK DATA.

	MOVF	DMD_DISP_D2+1,W		;COPY DMD_DISP_D2+1 TO W.
	DMD_D2_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D2_LATCH			;LATCH 373 TO LOCK DATA.

	MOVF	DMD_DISP_D3+1,W		;COPY DMD_DISP_D3+1 TO W.
	DMD_D3_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D3_LATCH			;LATCH 373 TO LOCK DATA.

	BSF	DMD_ROW2		;TURN ON ROW 2
	RETURN

;------------------------------------------------------------------------------
; UPDATE ROW 3 THE DISPLAY
;------------------------------------------------------------------------------
DMD_ROW3_UPDATE
	BANKSEL	DMD_ROW_PORT
	MOVF	DMD_DISP_D1+2,W		;COPY DMD_DISP_D1+2 TO W.
	BCF	DMD_ROW2		;TURN OFF ROW 2.
	DMD_D1_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D1_LATCH			;LATCH 373 TO LOCK DATA.

	MOVF	DMD_DISP_D2+2,W		;COPY DMD_DISP_D2+2 TO W.
	DMD_D2_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D2_LATCH			;LATCH 373 TO LOCK DATA.

	MOVF	DMD_DISP_D3+2,W		;COPY DMD_DISP_D3+2 TO W.
	DMD_D3_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D3_LATCH			;LATCH 373 TO LOCK DATA.

	BSF	DMD_ROW3		;TURN ON ROW 3.
	RETURN

;------------------------------------------------------------------------------
; UPDATE ROW 4 THE DISPLAY
;------------------------------------------------------------------------------
DMD_ROW4_UPDATE
	BANKSEL	DMD_ROW_PORT
	MOVF	DMD_DISP_D1+3,W		;COPY DMD_DISP_D1+3 TO W.
	BCF	DMD_ROW3		;TURN OFF ROW 3.
	DMD_D1_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D1_LATCH			;LATCH 373 TO LOCK DATA.

	MOVF	DMD_DISP_D2+3,W		;COPY DMD_DISP_D2+3 TO W.
	DMD_D2_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D2_LATCH			;LATCH 373 TO LOCK DATA.

	MOVF	DMD_DISP_D3+3,W		;COPY DMD_DISP_D3+3 TO W.
	DMD_D3_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D3_LATCH			;LATCH 373 TO LOCK DATA.

	BSF	DMD_ROW4		;TURN ON ROW 4.
	RETURN

;------------------------------------------------------------------------------
; UPDATE ROW 5 THE DISPLAY
;------------------------------------------------------------------------------
DMD_ROW5_UPDATE
	BANKSEL	DMD_ROW_PORT
	MOVF	DMD_DISP_D1+4,W		;COPY DMD_DISP_D1+4 TO W.
	BCF	DMD_ROW4		;TURN OFF ROW 4.
	DMD_D1_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D1_LATCH			;LATCH 373 TO LOCK DATA.

	MOVF	DMD_DISP_D2+4,W		;COPY DMD_DISP_D2+4 TO W.
	DMD_D2_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D2_LATCH			;LATCH 373 TO LOCK DATA.

	MOVF	DMD_DISP_D3+4,W		;COPY DMD_DISP_D3+4 TO W.
	DMD_D3_SET			;UNLOCK 373 REGISTER SO WE CAN LOAD DATA.
	CALL	DMD_DATA_SHOW		;MOVE DATA TO DISPLAY DATA PORT.
	DMD_D3_LATCH			;LATCH 373 TO LOCK DATA.

	BSF	DMD_ROW5		;TURN ON ROW 5.
	RETURN

;------------------------------------------------------------------------------
; PLACE THE CORRECT DATA ON THE DISPLAY PORT.
;------------------------------------------------------------------------------
DMD_DATA_SHOW
	MOVWF	DMD_TEMP		;COPY W TO DMD_TEMP.

	BTFSS	DMD_D1_BIT		;BIT 7 IS USED TO CONTROL D1 373 IC.
	BCF	DMD_TEMP,7		;TEST BIT ON PORT AND SET IT CORRECTLY
	BTFSC	DMD_D1_BIT		;IN DMD_TEMP SO WHEN WE MOVE IT TO 
	BSF	DMD_TEMP,7		;DATA PORT, BIT ISN'T CHANGED.

	MOVF	DMD_TEMP,W		;COPY DMD_TEMP TO W.
	BANKSEL	DMD_DATA_PORT
	MOVWF	DMD_DATA_PORT		;COPY DATA TO DATA PORT.
	RETURN

;------------------------------------------------------------------------------
; CHARACTER DATA. THE FOLLOWING ROUTINE CONTAINS HARDCODED INSTRUCTION TO
; GENERATE THE NUMBER '10' ON THE DISPLAY. 
;------------------------------------------------------------------------------
DMD_CHAR1
	MOVLW	B'00100111'		;CODE TO DISPLAY "10" ON FIRST DIGIT.
	MOVWF	DMD_DISP_D1+4
	MOVLW	B'01100101'
	MOVWF	DMD_DISP_D1+3
	MOVLW	B'00100101'
	MOVWF	DMD_DISP_D1+2
	MOVLW	B'00100101'
	MOVWF	DMD_DISP_D1+1
	MOVLW	B'01110111'
	MOVWF	DMD_DISP_D1
	
	MOVLW	B'01111111'		;CODE TO DISPLAY UPSIDE DOWN TRIANGLE
	MOVWF	DMD_DISP_D2+4
	MOVLW	B'00111110'
	MOVWF	DMD_DISP_D2+3
	MOVLW	B'00011100'
	MOVWF	DMD_DISP_D2+2
	MOVLW	B'00001000'
	MOVWF	DMD_DISP_D2+1
	MOVLW	B'00000000'
	MOVWF	DMD_DISP_D2

	MOVLW	B'00000000'		;CODE TO DISPLAY RIGHTSIDE UP TRIANGLE.
	MOVWF	DMD_DISP_D3+4
	MOVLW	B'00001000'
	MOVWF	DMD_DISP_D3+3
	MOVLW	B'00011100'
	MOVWF	DMD_DISP_D3+2
	MOVLW	B'00111110'
	MOVWF	DMD_DISP_D3+1
	MOVLW	B'01111111'
	MOVWF	DMD_DISP_D3
	RETURN



          END